package se.su.dsv.scipro.daisyExternal;

import brave.Tracing;
import brave.http.HttpTracing;
import com.google.inject.Provides;
import com.google.inject.Singleton;
import se.su.dsv.scipro.daisyExternal.http.DaisyAPI;
import se.su.dsv.scipro.daisyExternal.http.DaisyAPIImpl;
import se.su.dsv.scipro.daisyExternal.impl.ExternalImporterDaisyImpl;
import se.su.dsv.scipro.daisyExternal.impl.ImporterTransactionsImpl;
import se.su.dsv.scipro.util.PrivatePropertyModule;
import zipkin.Span;
import zipkin.reporter.AsyncReporter;
import zipkin.reporter.Reporter;
import zipkin.reporter.urlconnection.URLConnectionSender;

import javax.inject.Named;
import java.util.Properties;

public class DaisyExternalModule extends PrivatePropertyModule {
    @Override
    protected void configure(Properties properties) {
        bind(ExternalImporter.class).to(ExternalImporterDaisyImpl.class);
        bind(ImporterTransactions.class).to(ImporterTransactionsImpl.class);
        bind(DaisyAPI.class).to(DaisyAPIImpl.class);

        bindProperty(properties, "url");
        bindProperty(properties, "user");
        bindProperty(properties, "password");
        bindProperty(properties, "zipkin.endpoint");

        expose(ExternalImporter.class);
        expose(ImporterTransactions.class);
        expose(DaisyAPI.class);
    }

    @Provides
    @Singleton
    public HttpTracing httpTracing(final Reporter<Span> reporter) {
        final Tracing tracing = Tracing.newBuilder()
                .traceId128Bit(false)
                .localServiceName("scipro")
                .reporter(reporter)
                .build();
        return HttpTracing.newBuilder(tracing)
                .clientParser(new ExtendedHttpClientParser())
                .build();
    }

    @Provides
    @Singleton
    private Reporter<Span> getReporter(@Named("zipkin.endpoint") final String endpoint) {
        if (endpoint == null || endpoint.isEmpty() || endpoint.equalsIgnoreCase("${zipkin.endpoint}")) {
            return Reporter.NOOP;
        }
        else {
            return AsyncReporter.create(URLConnectionSender.create(endpoint));
        }
    }
}
